/*  aqotdd - Afghan's Quote of the day server - An RFC865 compliant server.
    Copyright (C) 2000 Afghan Hound Software

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA*/

/* signal.c - sets up signal handlers and deals with signals */

#include "main.h"

RETSIGTYPE signal_alarm (int sig);
RETSIGTYPE signal_fatal(int sig);
RETSIGTYPE signal_hup(int sig);
RETSIGTYPE signal_usr1(int sig);

volatile sig_atomic_t signal_fatal_running = 0;
int   signal_hup_received;
int   signal_usr1_received;

RETSIGTYPE signal_fatal (int sig)
{
  if (signal_fatal_running)
    {
      exit(1);
    }
  signal_fatal_running = 1;
  if (use_tcp)
    {
      close(listen_fd_tcp);
    }
  if (use_udp)
    {
      close(listen_fd_udp);
    }
  exit(1);
}

RETSIGTYPE signal_hup (int sig)
{
  signal_hup_received = 1;
}

RETSIGTYPE signal_usr1 (int sig)
{ 
  signal_usr1_received  = 1;
}

RETSIGTYPE signal_alarm (int sig)
{
  got_alarm = 1;
  if (mark_time)
    {
      alarm(mark_time);
    }
}

void signal_check()
{
  int              time_now;
  static int       time_last=0;
#ifdef MEMDEBUG
  struct mallinfo *meminfo;
#endif /* MEMDEBUG */
  if (!time_last)
    {
      time_last=time(NULL);
    }
  if (signal_hup_received)
    {
      my_message(MSG_SIGHUP,"ReReading Config.","");
      config_readfile();
      net_build_control_lists();
      qotd_init();
      my_message(MSG_SIGHUP,"ReRead Config.","");
      signal_hup_received = 0;
    }
  if (signal_usr1_received)
    {
#ifdef MEMDEBUG
      meminfo = (struct mallinfo *)xmalloc(sizeof(struct mallinfo));
      *meminfo = mallinfo();
      log_message(LOG_INFO,"Flag/Port/Max Connect (TCP/UDP), Flags: offensive/rfc/syslog/daemon, Log Level(log/connect), Directory, Log file, User, Log time, Order/Allow/Deny, Mem Stats(Arena/Chunk Free/Blocks In use/Blocks Free): %d %d %d %d %d %d, %d %d %d %d, %d %d, %s, %s, %s, %d, %d/%s/%s (%d %d %d %d)\n",use_tcp,use_udp,real_tcp_port,real_udp_port,max_con_tcp,max_con_udp,use_off,use_rfc,use_sys,daemon_flag,log_msg,log_con,qotd_dir,log_file,def_user,mark_time,order,allow,deny,meminfo->arena,meminfo->ordblks,meminfo->uordblks,meminfo->fordblks);
      free(meminfo);
#else 
      log_message(LOG_INFO,"Flag/Port/Max Connect (TCP/UDP), Flags: offensive/rfc/syslog/daemon, Log Level(log/connect), Directory, Log file, User, Log time, Order/Allow/Deny: %d %d %d %d %d %d, %d %d %d %d, %d %d, %s, %s, %s, %d, %d/%s/%s\n",use_tcp,use_udp,real_tcp_port,real_udp_port,max_con_tcp,max_con_udp,use_off,use_rfc,use_sys,daemon_flag,log_msg,log_con,qotd_dir,log_file,def_user,mark_time,order,allow,deny);
#endif /* MEMDEBUG */
      signal_usr1_received = 0;
    }
  if (got_alarm)
    {
      if(log_count)
	{
	  /* Don't waste time with hundreds of '0' reports */
	  if ((udp_count)||(tcp_count))
	    {
	      time_now = time(NULL);
	      log_message(LOG_INFO,"Counts: %d(udp),%d(tcp). Reallocs: %d(1st),%d(repeat): Last Stat %d(seconds).\n",udp_count,tcp_count,realloc1,realloc2,(time_now-time_last));
	      udp_count = 0;
	      tcp_count = 0;
	      realloc1  = 0;
	      realloc2  = 0;
	      time_last = time_now;
	    }
	}
      if (log_fileh!=NULL)
	{
	  fflush(log_fileh);
	}
      else
	{
	  fflush(stderr);
	}
      got_alarm=0;
    }
}

void signal_init()
{
  struct sigaction sig;
  my_message(MSG_INITSTART,"Starting Signal Handlers.","");

  signal (SIGINT, signal_fatal);
  signal (SIGTERM,signal_fatal);
  signal (SIGSEGV,signal_fatal);
;
  
  sig.sa_handler = SIG_IGN;
  sig.sa_flags = 0;
  sigemptyset(&sig.sa_mask);
  sigaction(SIGPIPE,&sig,NULL);

  sig.sa_handler = signal_hup;
  sig.sa_flags = SA_RESTART;
  sigemptyset(&sig.sa_mask);
  sigaction(SIGHUP,&sig,NULL);
  
  sig.sa_handler = signal_usr1;
  sig.sa_flags = SA_RESTART;
  sigemptyset(&sig.sa_mask);
  sigaction(SIGUSR1,&sig,NULL);

  if (mark_time)
    {
      sig.sa_handler = signal_alarm;
      sig.sa_flags = SA_RESTART;
      sigemptyset(&sig.sa_mask);
      sigaction(SIGALRM,&sig,NULL);
      alarm(mark_time);
    }
  my_message(MSG_INITFINISH,"Signal Handlers started.","");
}
